@@ -26,24 +26,20 @@ def from_json_dict(root):
 
 
 def register_definition(
- 			name=None,
-			aliases=set(),
-			description='',
-			data_type=DataType.String,
-			data_format=None,
-			default_value=Utils.DataDefinitionDefaultValue(),
-			examples=None,
-			nullable=False,
-			deprecated=False,
-			is_internal=False,
-			rules=[],
-			meta=None,
-			dependency_resolver=None
+ 			name,
+			aliases=Utils.Empty,
+			description=Utils.Empty,
+			data_type=Utils.Empty,
+			data_format=Utils.Empty,
+			default_value=Utils.Empty,
+			examples=Utils.Empty,
+			nullable=Utils.Empty,
+			deprecated=Utils.Empty,
+			internal=Utils.Empty,
+			rules=Utils.Empty,
+			meta=Utils.Empty,
+			dependency_resolver=Utils.Empty
 		):
-	if meta is None:
-		meta = Definitions.DefaultMeta()
-	if dependency_resolver is None:
-		dependency_resolver = Definitions.DefaultResolver()
 	result = Definitions.Index.register_definition(
 			name=name,
 			aliases=aliases,
@@ -54,7 +50,7 @@ def register_definition(
 			examples=examples,
 			nullable=nullable,
 			deprecated=deprecated,
-			is_internal=is_internal,
+			internal=is_internal,
 			rules=rules,
 			meta=meta,
 			dependency_resolver=dependency_resolver
@@ -80,27 +76,11 @@ def list_fields(name_or_definition):
 	definition = name_or_definition
 	if not isinstance(definition, Definitions.Definition):		
 		definition = get(str(name_or_definition))
-	if definition.data_type == DataType.Object:
-		field_names = definition.validator.list_field_names()
-		result = {
-				k: {
-						'definition_name': definition.validator.model_validator.get_field_definition_name(k),
-						'input_name': definition.validator.model_validator.get_field_input_name(k),
-						'input_display_name': definition.validator.model_validator.get_field_input_display_name(k),
-					}
-				for k in field_names
-			}
-		return result
-	return {}
+	return definition.list_fields()
 
 
-def validate_inputs(validated, **kw):
-	results = Definitions.Index.validate_inputs(validated, **kw)
-	return results
-
-
-def validate_input(definition_or_name, value, validated=None, field_fqn=None, field_name=None, dependent_values={}):
-	results = Definitions.Index.validate_input(validated, field_fqn, field_name, definition_or_name, value, dependent_values)
+def validate_input(definition_or_name, value, field_fqn=None, field_name=None, dependent_values={}):
+	results = Definitions.Index.validate_input(None, field_fqn, field_name, definition_or_name, value, dependent_values)
 	return results
 
 
