@@ -7,21 +7,23 @@ import cro_validate.classes.parameter_classes as Parameters
 import cro_validate.classes.schema_classes as Schemas
 import cro_validate.classes.name_strategy_classes as NameStrategies
 import cro_validate.classes.util_classes as Utils
+import cro_validate.classes.dependency_classes as Dependencies
 
 
 class Meta:
 	def initialize(self, definition, **kw):
 		raise NotImplementedError()
 
-	def to_json_dict(self):
-		return {}
 
+class DefaultDefinitionMeta(Meta):
+	component_name_strategy = NameStrategies.DefaultComponentNameStrategy()
+	schema_name = None
+	component_name = None
 
-class DefaultMeta(Meta):
-	def __init__(self, component_name_strategy=NameStrategies.DefaultComponentNameStrategy()):
-		self.component_name_strategy = component_name_strategy
-		self.schema_name = None
-		self.component_name = None
+	def __init__(self, component_name_strategy=Utils.Empty):
+		if not Utils.Empty.isempty(component_name_strategy):
+			self.component_name_strategy = component_name_strategy
+		
 
 	def initialize(self, definition, component_name_suffix='Model', display_name=None):
 		if definition.is_object():
@@ -32,28 +34,32 @@ class DefaultMeta(Meta):
 		else:
 			self.component_name = self.component_name_strategy.create_name(definition, component_name_suffix, display_name)
 
-		def to_json_dict(self):
-			return {
-				'component_name_strategy': self.component_name_strategy.to_json_dict()
-			}
-
 
 class Definition:
-	'''Def
-	'''
+	name = None
+	aliases = []
+	description = ''
+	data_type = DataType.String
+	data_format = None
+	default_value = Utils.Empty
+	examples = None
+	nullable = False
+	deprecated = False
+	internal = False
+	rules = []
+	meta = DefaultDefinitionMeta()
+	dependency_resolver = Dependencies.DefaultDefinitionDependencyResolver()
 
 	def to_json_dict(self):
-		default_value = self.default_value
-		if self.has_default_value() is False:
-			default_value = None
-		default_value_type = Utils.ClassName.class_fqn(self.default_value)
-		meta = None
-		if self.meta is not None:
-			meta = self.meta.to_json_dict()
-		meta_type = Utils.ClassName.class_fqn(self.meta)
-		dependency_resolver = None
-		if self.dependency_resolver is not None:
-			dependency_resolver = self.dependency_resolver.to_json_dict()
+		def _is_default(name, value):
+			class_value = getattr(self.__class__, name)
+			if class_value == value:
+				return True
+			return False
+		def _set(target, name, value):
+			if _is_default(name, value):
+				return
+			target[name] = value		
 		dependency_resolver_type = Utils.ClassName.class_fqn(self.dependency_resolver)
 		fmt = None
 		if self.data_format is not None:
@@ -67,27 +73,30 @@ class Definition:
 			else:
 				fmt = self.data_format
 		fmt_type = Utils.ClassName.class_fqn(self.data_format)
+		default_value_type = Utils.ClassName.class_fqn(self.default_value)
 		aliases = [a for a in self.aliases]
 		aliases.sort()
-		return {
-				#'name': self.name,
-				'aliases': aliases,
-				'description': self.description,
-				'data_type': self.data_type.name,
-				'data_format': fmt,
-				'data_format_type': fmt_type,
-				'default_value': default_value,
-				'default_value_type': default_value_type,
-				'examples': self.examples,
-				'nullable': self.nullable,
-				'deprecated': self.deprecated,
-				'is_internal': self.is_internal,
-				'rules': [{'type':Utils.ClassName.class_fqn(r), 'config':r.to_json_dict()} for r in self.rules],
-				'meta': meta,
-				'meta_type': meta_type,
-				'dependency_resolver': dependency_resolver,
-				'dependency_resolver_type': dependency_resolver_type
+		result = {}
+		_set(result, 'aliases', aliases)
+		_set(result, 'description', self.description)
+		_set(result, 'data_type', self.data_type.name)
+		_set(result, 'data_format', fmt)
+		_set(result, 'default_value', self.default_value)
+		_set(result, 'examples', self.examples)
+		_set(result, 'nullable', self.nullable)
+		_set(result, 'deprecated', self.deprecated)
+		_set(result, 'internal', self.internal)
+		_set(result, 'rules', [{'type':Utils.ClassName.class_fqn(r), 'config':r.to_json_dict()} for r in self.rules])
+		types = {
+				'default_value': default_value_type,
+				'data_format': fmt_type
 			}
+		result = {k: result[k] for k in result if not _is_default(k, result[k])}
+		for k in types:
+			t = types[k]
+			if k in result:
+				result[k + '_type'] = t
+		return result
 
 	def __init__(
 				self,
@@ -100,25 +109,29 @@ class Definition:
 				examples,
 				nullable,
 				deprecated,
-				is_internal,
+				internal,
 				rules,
 				meta,
 				dependency_resolver
 			):
-		self.name = name
-		self.aliases = aliases
-		self.description = description
-		self.data_type = data_type
-		self.data_format = data_format
-		self.default_value = default_value
-		self.examples = examples
-		self.nullable = nullable
-		self.deprecated = deprecated
-		self.is_internal = is_internal
-		self.dependencies = set()
-		self.rules = rules
-		self.meta = meta
-		self.dependency_resolver = dependency_resolver
+		def _set_member(name, value):
+			if Utils.Empty.isempty(value):
+				return
+			setattr(self, name, value)
+		_set_member('name', name)
+		_set_member('aliases', aliases)
+		_set_member('description', description)
+		_set_member('data_type', data_type)
+		_set_member('data_format', data_format)
+		_set_member('default_value', default_value)
+		_set_member('examples', examples)
+		_set_member('nullable', nullable)
+		_set_member('deprecated', deprecated)
+		_set_member('internal', internal)
+		_set_member('dependencies', set())
+		_set_member('rules', rules)
+		_set_member('meta', meta)
+		_set_member('dependency_resolver', dependency_resolver)
 		# Name
 		######
 		self.name = Config.definition_name_strategy.create_name(self, self.name)
@@ -136,16 +149,16 @@ class Definition:
 			self.nullable = True
 		# Default Value
 		###############
-		if isinstance(self.default_value, Utils.DataDefinitionDefaultValue):
+		if Utils.Empty.isempty(self.default_value):
 			if self.nullable is True:
 				self.default_value = None
 		# Data Format
 		#############
-		if self.data_type is DataType.Object and isinstance(self.data_format, str):
+		if self.is_object() and isinstance(self.data_format, str):
 			format_definition = Index.get(self.data_format)
 			self.data_format = format_definition.data_format
 		elif self.data_type is DataType.OneOf:
-			self.data_format = dependency_resolver.list_dependent_definition_names(self.name)
+			self.data_format = self.dependency_resolver.list_dependent_definition_names(self.name)
 		# Validator
 		###########
 		if self.is_object():
@@ -186,17 +199,7 @@ class Definition:
 		output_names = {}
 		dependencies = {}
 		default_values = {}
-		model = self.data_format.model
-		if model is None:
-			raise Config.exception_factory.create_internal_error(self.data_format.model_name, 'Missing model')
-		fields = {}
-		if isinstance(model, dict):
-			fields = model
-		else:
-			for name in dir(model):
-				if name.startswith('_'):
-					continue
-				fields[name] = getattr(model, name)
+		fields = self.list_fields()
 		for name in fields:
 			field_definition = fields[name]
 			if field_definition is None:
@@ -215,7 +218,7 @@ class Definition:
 				output_names[name] = field_definition.output_name
 			if field_definition.input_name:
 				input_names[name] = field_definition.input_name
-			if not isinstance(field_definition.default_value, Utils.DataDefinitionDefaultValue):
+			if not Utils.Empty.isempty(field_definition.default_value):
 				default_values[name] = field_definition.default_value
 			definition_name = name
 			if field_definition.definition_name is not None:
@@ -261,6 +264,7 @@ class Definition:
 		The validate func
 		'''
 		try:
+			results = Parameters.Index.ensure(results)
 			if not self.validator:
 				raise Config.exception_factory.create_internal_error(self.name, "Missing validator.")
 			if field_name is None:
@@ -269,7 +273,6 @@ class Definition:
 				field_fqn = field_name
 				if self.data_type == DataType.Object:
 					field_fqn = self.validator.model_validator.name
-			normalized = Parameters.Index()
 			dependent_definition_name = self.dependency_resolver.get_dependent_definition(
 					field_fqn,
 					dependent_values
@@ -290,18 +293,18 @@ class Definition:
 						dependent_values
 					)
 			else:
-				self.validator(normalized, field_fqn, field_name, self, value, dependent_values)
+				self.validator(results, field_fqn, field_name, self, value, dependent_values)
 			for rule in self.rules:
-				normalized[field_name] = rule.execute(field_fqn, normalized[field_name])
-			results.update(normalized)
+				results[field_name] = rule.execute(field_fqn, results[field_name])
+			return results
 		except Exception as ex:
-			if self.is_internal:
+			if self.internal:
 				raise Config.exception_factory.create_internal_error(ex.source, ex.message)
 			else:
 				raise ex
 
 	def has_default_value(self):
-		if isinstance(self.default_value, Utils.DataDefinitionDefaultValue):
+		if Utils.Empty.isempty(self.default_value):
 			return False
 		return True
 
@@ -340,153 +343,64 @@ class Definition:
 	def is_internal(self):
 		return self.is_internal
 
-class DependentDefinitionResolver:
-	def list_dependent_definition_names(self, fqn):
-		raise NotImplementedError()
+	def is_nullable(self):
+		return self.nullable
 
-	def get_dependent_definition(self, fqn, dependent_values):
-		raise NotImplementedError()
+	def list_fields(self):
+		if self.data_type is not DataType.Object:
+			return {}
+		model = self.data_format.model
+		if model is None:
+			raise Config.exception_factory.create_internal_error(self.data_format.model_name, 'Missing model')
+		fields = {}
+		if isinstance(model, dict):
+			fields = model
+		else:
+			for name in dir(model):
+				if name.startswith('_'):
+					continue
+				fields[name] = getattr(model, name)
+		for k in fields:
+			f = fields[k]
+			if f.input_name is None:
+				f.input_name = k
+		return fields
 
-	def list_dependency_fields(self, fqn):
-		raise NotImplementedError()
-
-
-class DefaultResolver(DependentDefinitionResolver):
-	def list_dependent_definition_names(self, fqn):
-		return []
-
-	def to_json_dict(self):
-		return {}
-
-	def get_dependent_definition(self, fqn, dependent_values):
-		return None
-
-	def list_dependency_fields(self, fqn):
-		return []
-
-	def to_json_dict(self):
-		return {}
-
-
-class _OneOfResolverPlayback:
-	def __init__(self, fqn, dependency_state, dependent_definition_name):
-		self.fqn = fqn
-		self.dependency_state = dependency_state
-		self.dependent_definition_name = dependent_definition_name
-
-	def to_json_dict(self):
-		return {
-				'fqn': self.fqn,
-				'dependency_state': self.dependency_state,
-				'dependent_definition_name': self.dependent_definition_name
-			}
-
-
-class OneOfResolver(DependentDefinitionResolver):
-	def __init__(self, playback=[]):
-		self._dependencies = set()
-		self._dependency_idx = {}
-		self._dependency_order = []
-		self._permutations = []
-		self._serialize_playback = []
-		for entry in playback:
-			self.index_dependent_definition(
-					entry['fqn'],
-					entry['dependency_state'],
-					entry['dependent_definition_name']
-				)
-
-	def to_json_dict(self):
-		return {
-				'playback': [entry.to_json_dict() for entry in self._serialize_playback]
-			}
-
-	def _update_dependency_idx_order(self, fqn, keys):
-		diff = self._dependencies.difference(keys)
-		if len(diff) > 0:
-			raise Config.exception_factory.create_internal_error(fqn, 'Permutation keys must match dependencies.')
-		self._dependencies.update([k for k in keys])
-		for k in keys:
-			if k in self._dependency_order:
-				continue
-			self._dependency_order.append(k)
-		self._dependency_order.sort()
-
-	def _index_permutation(self, fqn, permutation, value):
-		self._permutations.append(permutation)
-		idx = self._dependency_idx
-		for k in self._dependency_order[:-1]:
-			state = permutation[k]
-			if state not in idx:
-				idx[state] = {}
-			idx = idx[state]
-		state = permutation[self._dependency_order[-1]]
-		if state in idx:
-			raise Config.exception_factory.create_internal_error(fqn, 'Cannot re-index a permutation.')
-		idx[state] = value
-
-	def _get_permutation_value(self, fqn, permutation):
-		idx = self._dependency_idx
-		last_index = len(self._dependency_order) - 1
-		for i in range(len(self._dependency_order)):
-			k = self._dependency_order[i]
-			if k not in permutation:
-				raise Config.exception_factory.create_internal_error(
-						fqn,
-						'k not in permutation ({0})'.fromat(permutation)
-					)
-			state = permutation[k]
-			if state not in idx:
-				raise Config.exception_factory.create_internal_error(
-						fqn,
-						'Unknown permutation value (k={0} p={2}).'.format(k, state, permutation)
-					)
-			if i == last_index:
-				return idx[state]
-			idx = idx[state]
-		return None
-
-	def list_dependent_definition_names(self, fqn):
-		result = set()
-		for p in self._permutations:
-			name = self._get_permutation_value(fqn, p)
-			result.add(name)
+	def get_boundaries(self):
+		result = []
+		for rule in self.rules:
+			result.extend(rule.get_boundaries())
+		if self.is_nullable() is not True:
+			result.append(None)
+		if self.is_object() is True:
+			result.append([])
+		if self.is_array() is True:
+			result.append({})
 		return result
 
-	def index_dependent_definition(self, fqn, dependency_state, dependent_definition_name):
-		self._update_dependency_idx_order(fqn, dependency_state.keys())
-		self._index_permutation(fqn, dependency_state, dependent_definition_name)
-		self._serialize_playback.append(_OneOfResolverPlayback(fqn, dependency_state, dependent_definition_name))
-
-	def get_dependent_definition(self, fqn, dependent_values):
-		name = self._get_permutation_value(fqn, dependent_values)
-		return name
-
-	def list_dependency_fields(self, fqn):
-		return self._dependencies
-
 
 class DefinitionJsonDeserializer:
-	def _get_dict_value(self, idx, k, default_value=Utils.DataDefinitionDefaultValue()):
+	def _get_dict_value(self, idx, k, default_value=Utils.Empty):
 		if k in idx:
 			return idx[k]
 		return default_value
 
-	def _set(self, src, tgt, k, default_value=Utils.DataDefinitionDefaultValue()):
-			v = self._get_dict_value(src, k, default_value)
-			if isinstance(v, Utils.DataDefinitionDefaultValue):
-				return
-			tgt[k] = v
+	def _set(self, src, tgt, k, default_value=Utils.Empty):
+		v = self._get_dict_value(src, k, default_value)
+		if Utils.Empty.isempty(v):
+			return
+		if k == 'is_internal':
+			k = 'internal'
+		tgt[k] = v
 
 	def _deserialize_schema_field(self, root, name, obj):
 		kw = {}
-		if obj['default_value_type'] != 'cro_validate.classes.util_classes.DataDefinitionDefaultValue':
-			kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
-		else:
-			kw['default_value'] = Utils.DataDefinitionDefaultValue()
-		kw['definition_name'] = obj['definition_name']
-		if kw['definition_name'] is None:
-			kw['definition_name'] = name
+		if 'default_value' in obj:
+			if obj['default_value_type'] != 'cro_validate.classes.util_classes.Empty':
+				kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
+		kw['definition_name'] = name
+		if 'definition_name' in obj and obj['definition_name'] is not None:
+			kw['definition_name'] = obj['definition_name']
 		if not Index.exists(kw['definition_name']):
 			dependent_definition_profile = self.deserialize(root, kw['definition_name'])
 			Index.register_definition(**dependent_definition_profile)
@@ -496,22 +410,23 @@ class DefinitionJsonDeserializer:
 		self._set(obj, kw, 'output_name')
 		self._set(obj, kw, 'required')
 		self._set(obj, kw, 'unvalidated')
-		if isinstance(kw['dependencies'], list):
+		if 'dependencies' in kw and isinstance(kw['dependencies'], list):
 			kw['dependencies'] = set(kw['dependencies'])
 		field = Schemas.Field(**kw)
 		return field
 		
-	def _deserialize_schema(self, root, obj):
+	def _deserialize_schema(self, root, obj, name):
 		kw = {}
 		self._set(obj, kw, 'allow_unknown_fields')
 		self._set(obj, kw, 'case_sensitive')
 		self._set(obj, kw, 'display_name')
-		self._set(obj, kw, 'model_name')
 		self._set(obj, kw, 'model')
+		kw['model_name'] = name
 		model = {}
-		for name in obj['model']:
-			field = self._deserialize_schema_field(root, name, obj['model'][name])
-			model[name] = field
+		if 'model' in obj:
+			for name in obj['model']:
+				field = self._deserialize_schema_field(root, name, obj['model'][name])
+				model[name] = field
 		kw['model'] = model
 		schema = Schemas.Schema(**kw)
 		return schema
@@ -533,43 +448,56 @@ class DefinitionJsonDeserializer:
 		return rule
 
 	def deserialize(self, root, name):
-		kw = {}
+		kw = {
+			'name': Utils.Empty,
+			'aliases': Utils.Empty,
+			'description': Utils.Empty,
+			'examples': Utils.Empty,
+			'nullable': Utils.Empty,
+			'deprecated': Utils.Empty,
+			'internal': Utils.Empty,
+			'data_type': Utils.Empty,
+			'data_format': Utils.Empty,
+			'default_value': Utils.Empty,
+			'meta': Utils.Empty,
+			'dependency_resolver': Utils.Empty,
+			'rules': Utils.Empty
+		}
 		namespace = {k:k for k in root}
 		for k in root:
+			if 'aliases' not in root[k]:
+				continue
 			for k1 in root[k]['aliases']:
 				namespace[k1] = k
 		obj = root[namespace[name]]
 		# Data Format
 		#############
-		kw['data_type'] = DataType[obj['data_type']]
-		if obj['data_format_type'] == 'cro_validate.classes.schema_classes.Schema':
-			kw['data_format'] = self._deserialize_schema(root, obj['data_format'])
-		else:
-			kw['data_format'] = self._instantiate(obj['data_format_type'], obj['data_format'])
+		if 'data_type' in obj:
+			kw['data_type'] = DataType[obj['data_type']]
+		if 'data_format' in obj:
+			if obj['data_format_type'] == 'cro_validate.classes.schema_classes.Schema':
+				kw['data_format'] = self._deserialize_schema(root, obj['data_format'], name)
+			else:
+				kw['data_format'] = self._instantiate(obj['data_format_type'], obj['data_format'])
 		# Default Value
 		###############
-		if obj['default_value_type'] == 'builtins.NoneType':
-			kw['default_value'] = None
-		elif obj['default_value_type'] != 'cro_validate.classes.util_classes.DataDefinitionDefaultValue':
-			kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
-		else:
-			kw['default_value'] = Utils.DataDefinitionDefaultValue()
-		# Meta
-		######
-		kw['meta'] = self._instantiate(obj['meta_type'], **obj['meta'])
-		# Resolver
-		##########
-		kw['dependency_resolver'] = self._instantiate(obj['dependency_resolver_type'], **obj['dependency_resolver'])
+		if 'default_value' in obj:
+			if obj['default_value_type'] == 'builtins.NoneType':
+				kw['default_value'] = None
+			elif obj['default_value_type'] != 'cro_validate.classes.util_classes.Empty':
+				kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
 		# Rules
 		#######
-		rules = []
-		for rule in obj['rules']:
-			rule_obj = self._deserialize_rule(obj, rule)
-			rules.append(rule_obj)
-		kw['rules'] = rules
+		if 'rules' in obj:
+			rules = []
+			for rule in obj['rules']:
+				rule_obj = self._deserialize_rule(obj, rule)
+				rules.append(rule_obj)
+			kw['rules'] = rules
 		# Aliases
 		#########
-		kw['aliases'] = set(obj['aliases'])
+		if 'aliases' in obj:
+			kw['aliases'] = set(obj['aliases'])
 		# Simple
 		########
 		kw['name'] = name
@@ -630,7 +558,7 @@ class Index:
 				examples,
 				nullable,
 				deprecated,
-				is_internal,
+				internal,
 				rules,
 				meta,
 				dependency_resolver
@@ -645,7 +573,7 @@ class Index:
 				examples=examples,
 				nullable=nullable,
 				deprecated=deprecated,
-				is_internal=is_internal,
+				internal=internal,
 				rules=rules,
 				meta=meta,
 				dependency_resolver=dependency_resolver
@@ -653,7 +581,9 @@ class Index:
 		definition_name = definition.get_name()
 		names = set()
 		names.add(definition_name)
-		if isinstance(aliases, str):
+		if Utils.Empty.isempty(aliases):
+			pass
+		elif isinstance(aliases, str):
 			names.add(aliases)
 		else:
 			names.update(aliases)
@@ -664,12 +594,6 @@ class Index:
 			Index._idx[entry] = definition
 		return definition
 
-	def validate_inputs(validated, **kw):
-		results = Parameters.Index.ensure(validated)
-		for name in kw:
-			Index.validate_input(results, name, name, name, kw[name])
-		return results
-
 	def validate_input(validated, field_fqn, field_name, definition_or_name, value, dependent_values={}):
 		definition = Index.get(definition_or_name)
 		results = Parameters.Index.ensure(validated)
