@@ -3,6 +3,7 @@ import inspect
 import cro_validate.input.normalize_input as Normalize
 from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.definition_classes as Definitions
+from cro_validate.classes.util_classes import Boundaries
 
 
 class Rule:
@@ -10,11 +11,13 @@ class Rule:
 				self,
 				description,
 				callback=None,
-				config={}
+				config={},
+				boundaries=[]
 			):
 		self._description = description
 		self._callback = callback
 		self._config = config
+		self._boundaries = boundaries
 
 	def to_json_dict(self):
 		return self.get_config()
@@ -22,6 +25,9 @@ class Rule:
 	def get_config(self):
 		return self._config
 
+	def get_boundaries(self):
+		return self._boundaries
+
 	def get_description(self):
 		return self._description
 
@@ -73,6 +79,9 @@ class AsInt(Rule):
 				callback=Normalize.as_int
 			)
 
+	def get_boundaries(self):
+		return ['InvalidInt']
+
 
 class IntGte(Rule):
 	def __init__(self, minimum):
@@ -82,6 +91,9 @@ class IntGte(Rule):
 				config={'minimum':minimum}
 			)
 
+	def get_boundaries(self):
+		return [self.get_config()['minimum'] - 1]
+
 
 class IntLte(Rule):
 	def __init__(self, maximum):
@@ -91,6 +103,10 @@ class IntLte(Rule):
 				config={'maximum':maximum}
 			)
 
+	def get_boundaries(self):
+		bound = self.get_config()['maximum'] + 1
+		return [bound]
+
 
 class FloatGte(Rule):
 	def __init__(self, minimum):
@@ -100,6 +116,10 @@ class FloatGte(Rule):
 				config={'minimum':minimum}
 			)
 
+	def get_boundaries(self):
+		return [self.get_config()['minimum'] - 0.1]
+
+
 
 class FloatLte(Rule):
 	def __init__(self, maximum):
@@ -109,6 +129,10 @@ class FloatLte(Rule):
 				config={'maximum':maximum}
 			)
 
+	def get_boundaries(self):
+		bound = self.get_config()['maximum'] + 0.1
+		return [bound]
+
 
 class FloatLt(Rule):
 	def __init__(self, maximum):
@@ -118,19 +142,25 @@ class FloatLt(Rule):
 				config={'maximum':maximum}
 			)
 
+	def get_boundaries(self):
+		return [self.get_config()['maximum']]
+
 
 ########################################################################################################################
 #                                                       Strings                                                        #
 ########################################################################################################################
-
 class MatchRegex(Rule):
-	def __init__(self, rex, flags=0): # re.IGNORECASE
+	def __init__(self, rex, flags=0, boundaries=[]): # re.IGNORECASE
 		super().__init__(				
 				description='Must match expression {0}'.format(rex),
 				callback=Normalize.as_str_matching_regex,
-				config={'rex':rex, 'flags':flags}
+				config={'rex':rex, 'flags':flags},
+				boundaries=boundaries
 			)
 
+	def get_boundaries(self):
+		return self._boundaries
+
 
 class StrWithMinLen(Rule):
 	def __init__(self, min_len):
@@ -140,6 +170,10 @@ class StrWithMinLen(Rule):
 				config={'min_len':min_len}
 			)
 
+	def get_boundaries(self):
+		min_len = self.get_config()['min_len']
+		return Boundaries.get_str_min_len_boundary(min_len)
+
 
 class StrWithMaxLen(Rule):
 	def __init__(self, max_len):
@@ -149,6 +183,10 @@ class StrWithMaxLen(Rule):
 				config={'max_len':max_len}
 			)
 
+	def get_boundaries(self):
+		max_len = self.get_config()['max_len']
+		return Boundaries.get_str_max_len_boundary(max_len)
+
 
 class StrWithinInclusiveLenRange(Rule):
 	def __init__(self, min_len, max_len):
@@ -158,6 +196,15 @@ class StrWithinInclusiveLenRange(Rule):
 				config={'max_len':max_len, 'min_len':min_len}
 			)
 
+	def get_boundaries(self):
+		min_len = self.get_config()['min_len']
+		max_len = self.get_config()['max_len']
+		result = []
+		result.extend(Boundaries.get_str_min_len_boundary(min_len))
+		result.extend(Boundaries.get_str_max_len_boundary(max_len))
+		return result
+
+
 class ValueInSet(Rule):
 	def __init__(self, values):
 		desc = 'Must be one of: '
@@ -181,6 +228,15 @@ class ValueInSet(Rule):
 			config['values'] = list(config['values'])
 		return config
 
+	def get_boundaries(self):
+		s = '___not_in_set___'
+		values = self.get_config()['values']
+		if s in values:
+			raise Config.exception_factory.create_internal_error(
+					input_name,
+					'Invalid boundary configured: {0} in {1}.'.format(s, values))
+		return [s]
+
 
 	def list_values(self):
 		result = list(self.get_config()['values'])
