@@ -5,28 +5,43 @@ import cro_validate.classes.parameter_classes as Parameters
 import cro_validate.classes.util_classes as Utils
 
 class Schema:
+	model = None
+	model_name = None
+	allow_unknown_fields = False
+	display_name = None
+	case_sensitive = True
+
 	def __init__(
 				self,
 				model,
-				model_name=None,
-				allow_unknown_fields=False,
-				display_name=None,
-				case_sensitive=True
+				model_name=Utils.Empty,
+				allow_unknown_fields=Utils.Empty,
+				display_name=Utils.Empty,
+				case_sensitive=Utils.Empty
 			):
+		def _set_member(name, value):
+			if Utils.Empty.isempty(value):
+				return
+			setattr(self, name, value)
 		if isinstance(model, type):
-			self.model = model()
+			model = model()
 		elif isinstance(model, str):
-			self.model = Definitions.Index.get(model).data_format.model
-		else:
-			self.model = model
-		self.model_name = self.model.__class__.__name__
-		if model_name is not None:
-			self.model_name = model_name
-		self.allow_unknown_fields = allow_unknown_fields
-		self.display_name = display_name
-		self.case_sensitive = case_sensitive
+			model = Definitions.Index.get(model).data_format.model
+		_set_member('model', model)
+		_set_member('model_name', model_name)
+		_set_member('allow_unknown_fields', allow_unknown_fields)
+		_set_member('display_name', display_name)
+		_set_member('case_sensitive', case_sensitive)
 
 	def to_json_dict(self):
+		def _is_default(name, value):
+			if getattr(self.__class__, name) == value:
+				return True
+			return False
+		def _set(target, name, value):
+			if _is_default(name, value):
+				return
+			target[name] = value
 		model = Utils.ClassName.class_fqn(self.model),
 		model_type = 'DefinitionName'
 		if not isinstance(self.model, str):
@@ -45,68 +60,85 @@ class Schema:
 					elif field is None:
 						field = Field()
 						model[entry] = field.to_json_dict(entry)
-		return {
-					'model': model,
-					'model_name': self.model_name,
-					'model_type': model_type,
-					'allow_unknown_fields': self.allow_unknown_fields,
-					'display_name': self.display_name,
-					'case_sensitive': self.case_sensitive
-				}
+		result = {
+				'model_type': model_type
+			}
+		_set(result, 'model', model)
+		_set(result, 'model_name', self.model_name)
+		_set(result, 'allow_unknown_fields', self.allow_unknown_fields)
+		_set(result, 'display_name', self.display_name)
+		_set(result, 'case_sensitive', self.case_sensitive)
+		return result
 
 
 class Field:
+	input_name = None
+	output_name = None
+	definition_name = None
+	required = True
+	ignored = False
+	unvalidated = False
+	dependencies = set()
+	default_value = Utils.Empty
+
 	def __init__(
 				self,
-				input_name=None,
-				output_name=None,
-				definition_name=None,
-				required=True,
-				ignored=False,
-				unvalidated=False,
-				dependencies=set(),
-				default_value=Utils.DataDefinitionDefaultValue()):
-		self.input_name = input_name
-		self.output_name = output_name
-		self.definition_name = definition_name
-		self.required = required
-		self.ignored = ignored
-		self.unvalidated = unvalidated
-		self.dependencies = dependencies
-		self.default_value = default_value
+				input_name=Utils.Empty,
+				output_name=Utils.Empty,
+				definition_name=Utils.Empty,
+				required=Utils.Empty,
+				ignored=Utils.Empty,
+				unvalidated=Utils.Empty,
+				dependencies=Utils.Empty,
+				default_value=Utils.Empty):
+		def _set_member(name, value):
+			if Utils.Empty.isempty(value):
+				return
+			setattr(self, name, value)
+		_set_member('input_name', input_name)
+		_set_member('output_name', output_name)
+		_set_member('definition_name', definition_name)
+		_set_member('required', required)
+		_set_member('ignored', ignored)
+		_set_member('unvalidated', unvalidated)
+		_set_member('dependencies', dependencies)
+		_set_member('default_value', default_value)
 
-	def to_json_dict(self, name):
+	def to_json_dict(self, field_name):
+		def _is_default(name, value):
+			if getattr(self.__class__, name) == value:
+				return True
+			return False
+		def _set(target, name, value):
+			if _is_default(name, value):
+				return
+			target[name] = value
 		default_value = self.default_value
-		if isinstance(self.default_value, Utils.DataDefinitionDefaultValue):
+		if Utils.Empty.isempty(self.default_value):
 			default_value = None
-		default_value_type = Utils.ClassName.class_fqn(self.default_value)
 		definition_name = self.definition_name
-		if definition_name == name:
+		if definition_name == field_name:
 			definition_name = None
-		return {
-				'input_name': self.input_name,
-				'output_name': self.output_name,
-				'definition_name': definition_name,
-				'required': self.required,
-				'ignored': self.ignored,
-				'unvalidated': self.unvalidated,
-				'dependencies': [d for d in self.dependencies],
-				'default_value': default_value,
-				'default_value_type': default_value_type
-			}
+		result = {}
+		if self.input_name != field_name:
+			_set(result, 'input_name', self.input_name)
+		_set(result, 'output_name', self.output_name)
+		_set(result, 'definition_name', definition_name)
+		_set(result, 'required', self.required)
+		_set(result, 'ignored', self.ignored)
+		_set(result, 'unvalidated', self.unvalidated)
+		_set(result, 'dependencies', self.dependencies)
+		_set(result, 'default_value', self.default_value)
+		if 'default_value' in result:
+			default_value_type = Utils.ClassName.class_fqn(self.default_value)
+			result['default_value_type'] = default_value_type
+		return result
 
 
 class Validator:
 	def _validate(self, validated, fqn, field_name, definition, value, dependent_values):
 		results = Parameters.Index.ensure(validated)
-		normalized = value
-		if isinstance(value, dict):
-			normalized = Parameters.Index(value)
-		model_target = None
-		if not isinstance(value, dict):
-			# Get the original ref back
-			model_target = value
-		model_result = self.model_validator(model_target, fqn, field_name, definition, normalized)
+		model_result = self.model_validator(None, fqn, field_name, definition, value)
 		results[field_name] = model_result
 
 	def __init__(self, name, model_validator):
@@ -244,8 +276,8 @@ class ModelValidator:
 				return source[value_name]
 		elif hasattr(source, value_name):
 			return getattr(source, value_name)
-		if field_name in self.default_values:
-			return self.default_values[field_name]
+		#if field_name in self.default_values:
+		#	return self.default_values[field_name]
 		raise Config.exception_factory.create_input_error(fqn, 'Missing required value.')
 
 	def _get_output_value(self, value_name, source):
@@ -442,7 +474,7 @@ class ModelValidator:
 	def add_default_values(self, default_values):
 		for field_name in default_values:
 			value = default_values[field_name]
-			if isinstance(value, Utils.DataDefinitionDefaultValue):
+			if Utils.Empty.isempty(value):
 				continue
 			self.default_values[field_name] = value
 
